<?php
/*
Plugin Name: 123 Rel Url
Plugin URI: http://schnurpsel.de/wordpress-plugins/123-rel-url/
Description: Entfernt Protokoll und Hostname aus den URLs beim Einfügen von Bildern/Medien oder internen Links im Wordpress-Editor.
Author: Ingo Henze
Version: 0.10
Author URI: http://putzlowitsch.de/
*/
// direkten Aufruf der PHP-Datei unterbinden
if( !function_exists( 'is_admin' ) ) {
    header( 'Status: 403 Forbidden' );
    header( 'HTTP/1.1 403 Forbidden' );
    exit();
}

// relative URL erzeugen
function plw123ru_get_rel_url( $url ) {
	$url_parts = parse_url( $url );
	if( is_array( $url_parts ) ) {
		$relurl = $url_parts['path'];
		if( isset( $url_parts['query'] ) )
			$relurl .= '?'.$url_parts['query'];
		if( isset( $url_parts['fragment'] ) )
			$relurl .= '#'.$url_parts['fragment'];
		$url = $relurl;
	}
	return $url;
}

// Protokoll, Hostname und ggf. Port aus URL extrahieren
function plw123ru_get_host( $url ) {
	$url_parts = parse_url( $url );
	if( is_array( $url_parts ) ) {
		$host = '';
		if( isset( $url_parts['scheme'] ) )
			$host .= $url_parts['scheme'].'://';
		if( isset( $url_parts['host'] ) )
			$host .= $url_parts['host'];
		if( isset( $url_parts['port'] ) )
			$host .= ':'.$url_parts['port'];
		$url = $host;
	}
	return $url;
}

// Filter für Funktion 'the_content_rss'
function plw123ru_content_rss( $content ) {
	// Wird ein Feed ausgegeben?
	if( is_feed() )	{
		$plw123ru_hostname = plw123ru_get_host( home_url() );
		return preg_replace( "/<(a|img)(.+?)(href|src)=(\"|')\/(.+?)(\"|')/i", "<$1$2$3=$4{$plw123ru_hostname}/$5$6", $content );
	}
	else
		return $content;
}

// Filter für Funktion 'get_the_guid'
function plw123ru_guid_rss( $guid ) {
	// Wird ein Feed ausgegeben?
	if( is_feed() )
		if( $guid != '' ) {
			$plw123ru_hostname = plw123ru_get_host( home_url() );
			return $plw123ru_hostname.plw123ru_get_rel_url( $guid );
		}
	return $guid;
}

// relative interne Links und Bilder/Medien in RSS-Feeds "absolutieren"
add_filter( 'the_content', 'plw123ru_content_rss', 99 );
add_filter( 'get_comment_text', 'plw123ru_content_rss', 99 );
add_filter( 'get_the_guid', 'plw123ru_guid_rss' );

// WP-Fehler korrigieren (Port wird nicht mit übernommen)
function plw123ru_self_link( $link ) {
	$plw123ru_hostname = plw123ru_get_host( home_url() );
	return $plw123ru_hostname.wp_unslash( $_SERVER['REQUEST_URI'] );
}
add_filter( 'self_link', 'plw123ru_guid_rss' );

// In der Liste der internen Links relative URLs erzeugen
function plw123ru_wp_link_query( $results, $query ) {
	$new_results = array();
	foreach( $results as $data ) {
		$data['permalink'] = plw123ru_get_rel_url( $data['permalink'] );
		$new_results[] = $data;
	}
	return $new_results;
}
add_filter( 'wp_link_query', 'plw123ru_wp_link_query', 10, 2 );

// Beim Einfügen von Medien/Bildern relative URLs erzeugen
function plw123ru_media_send_to_editor( $html, $send_id, $attachment ) {
	// Liste der Namen von in WP verwendeten Link/Medien-Attributen
	$plw123ru_src_names = array_merge( array( 'src', 'href' ), wp_get_audio_extensions(), wp_get_video_extensions() );
	$plw123ru_hostname = preg_quote( plw123ru_get_host( home_url() ), '/' );
	$src = implode( '|', $plw123ru_src_names );
	return preg_replace( "/(<|\[)(.+?)($src)=(\"|')$plw123ru_hostname(.+?)(\"|')/i", "$1$2$3=$4$5$6", $html );
}
add_filter( 'media_send_to_editor', 'plw123ru_media_send_to_editor', 66, 3 );

// Base-URL im Header hinzufügen
function plw123ru_base_href() {
	$plw123ru_hostname = plw123ru_get_host( home_url() );
	echo "<base href='$plw123ru_hostname' />\n";
}
add_action( 'wp_head', 'plw123ru_base_href', 1 );
?>